
// bsc.h
//

#include <stdarg.h>

//////////////////////////////////////////////////////////////////////
// you must define the following callbacks for the library to use
// to avoid dependancy on the C standard io library.  If you don't
// define these then you accept the defaults which call C runtime
//

// malloc and free workalikes

LPV  BSC_API LpvAllocCb(WORD cb);
VOID BSC_API FreeLpv(LPV lpv);

// open, read, close, seek workalikes

int  BSC_API BSCOpen(LSZ lszFileName, int mode);
int  BSC_API BSCRead(int handle, LPCH lpchBuf, WORD cb);
int  BSC_API BSCSeek(int handle, long lPos, int mode);
int  BSC_API BSCClose(int handle);

// ascii text output routine

VOID BSC_API BSCOutput(LSZ lsz);

#ifdef DEBUG
VOID BSC_API BSCDebugOut(LSZ lsz);
VOID BSCDebug(LSZ lszFormat, ...);
#endif

// error handling routines
//
VOID BSC_API SeekError(LSZ lszFileName);	// (may choose to not return)
VOID BSC_API ReadError(LSZ lszFileName);	// (may choose to not return)
VOID BSC_API BadBSCVer(LSZ lszFileName);	// (may choose to not return)

// end of callbacks
//
///////////////////////////////////////////////////////////////////////

// an IDX is guaranteed to be big enough to hold any of the 
// database index types, i.e. it is a generic index

typedef DWORD IDX;

#define idxNil	 0xffffffffL
#define isymNil  (ISYM)(idxNil)
#define imodNil  (IMOD)(idxNil)
#define iinstNil (IINST)(idxNil)

// definition and prototypes for use with the bsc library
//
typedef WORD IMOD;
typedef WORD IMS;
typedef WORD ISYM;
typedef WORD IINST;
typedef DWORD IREF;
typedef WORD IDEF;
typedef DWORD IUSE;
typedef DWORD IUBY;
typedef WORD IBASE;
typedef WORD IDERV;
typedef WORD IFRIN;
typedef WORD IFROUT;

typedef WORD ATR;	// instance flags
typedef WORD TYP;	// instance flags

//  Open the specified data base.
//  Return TRUE iff successful, FALSE if database can't be read
//
BOOL BSC_API FOpenBSC (LSZ lszName);

// close database and free as much memory as possible
//
VOID BSC_API CloseBSC(VOID);

// return the length of the largest symbol in the database
//
WORD BSC_API BSCMaxSymLen(VOID);

// is this database built with a case sensitive language?
//
BOOL BSC_API FCaseBSC(VOID);

// override the case sensitivity of the database, symbol lookups become
// case (in)sensistive as specified
//
VOID BSC_API SetCaseBSC(BOOL fCaseSensitive);

// do a case insenstive compare qualified by a case sensitive compare
// if fCase is true -- this is the order of symbols in the symbol list
int BSC_API CaseCmp(LSZ lsz1, LSZ lsz2);

// nearly identical to CaseCmp except only the prefix is compared of the
// two strings
int BSC_API CaseCmpPrefix(LSZ lszprefix, LSZ lszdecor);

// return the name of the given symbol
//
LSZ BSC_API LszNameFrSym (ISYM isym);

// return the name of the given module
//
LSZ BSC_API LszNameFrMod (IMOD imod);

// return the imod with the given name -- imodNil if none
//
IMOD BSC_API ImodFrLsz(LSZ lszModName);

// return the isym with the given name -- isymNil if none
//
ISYM BSC_API IsymFrLsz(LSZ lszSymName);

// find the smallest ISYM whose value is greater or equal to the given LSZ
//
ISYM BSC_API IsymSupLsz(LSZ lszSymName);

// return the biggest isym in this database, isyms run from 0 to this value - 1
//
ISYM BSC_API IsymMac(VOID);

// return the biggest imod in this database, imods run from 0 to this value - 1
//
IMOD BSC_API ImodMac(VOID);

// return the biggest iinst in this database, iinsts run from 0 to the value -1
IINST BSC_API IinstMac(VOID);

// fill in the range of MS items valid for this module
//
VOID BSC_API MsRangeOfMod(IMOD imod, IMS FAR *pims, IMS FAR *pimsMac);

// give the instance index of the module symbol (MS)
//
IINST BSC_API IinstOfIms(IMS ims);

// fill in the range of inst values for this symbol
//
VOID BSC_API InstRangeOfSym(ISYM isym, IINST FAR *piinst, IINST FAR *piinstMac);

// get the information that qualifies this instance
//
VOID BSC_API InstInfo(IINST iinst, ISYM FAR *pisymInst, TYP FAR *ptyp, ATR FAR *patr);

// fill in the reference ranges from the inst
//
VOID BSC_API RefRangeOfInst(IINST iinst, IREF FAR *piref, IREF FAR *pirefMac);

// fill in the definition ranges from the inst
//
VOID BSC_API DefRangeOfInst(IINST iinst, IDEF FAR *pidef, IDEF FAR *pidefMac);

// fill in the use ranges from the inst
//
VOID BSC_API UseRangeOfInst(IINST iinst, IUSE FAR *piuse, IUSE FAR *piuseMac);

// fill in the used by ranges from the inst
//
VOID BSC_API UbyRangeOfInst(IINST iinst, IUBY FAR *piuby, IUBY FAR *piubyMac);

// fill in the base class ranges from the inst
//
VOID BSC_API BaseRangeOfInst(IINST iinst, IBASE FAR *pibase, IBASE FAR *pibaseMac);

// fill in the dervived class ranges from the inst
//
VOID BSC_API DervRangeOfInst(IINST iinst, IDERV FAR *piderv, IDERV FAR *pidervMac);

// fill in the incoming friend ranges from the inst
//
VOID BSC_API FrinRangeOfInst(IINST iinst, IFRIN FAR *pifrin, IFRIN FAR *pifrinMac);

// fill in the outgoing friend ranges from the inst
//
VOID BSC_API FroutRangeOfInst(IINST, IFROUT FAR *, IFROUT FAR *);

// fill in the information about this things which an inst uses
//
VOID BSC_API UseInfo(IUSE iuse, IINST FAR *piinst, WORD FAR *pcnt);

// fill in the information about this things which an inst is used by
//
VOID BSC_API UbyInfo(IUBY iuby, IINST FAR *piinst, WORD FAR *pcnt);

// get the information about a base class (mode is inheritance mode)
//
VOID BSC_API BaseInfo(IBASE ibase, IINST FAR *piinst, WORD FAR *pmode);

// get the information about a derived class (mode is inheritance mode)
//
VOID BSC_API DervInfo(IDERV iderv, IINST FAR *piinst, WORD FAR *pmode);

// get the information about an incoming friend
//
VOID BSC_API FrinInfo(IFRIN ifrin, IINST FAR *piinst);

// get the information about an outgoing friend
//
VOID BSC_API FroutInfo(IFROUT ifrout, IINST FAR *piinst);

// fill in the information about this reference
//
VOID BSC_API RefInfo(IREF iref, LSZ FAR *plszName, WORD FAR *pline);

// fill in the information about this definition
//
VOID BSC_API DefInfo(IDEF idef, LSZ FAR *plszName, WORD FAR *pline);

// these are the bit values for the InstInfo() call InstFlags WORD
//
//

// this is the type part of the field, it describes what sort of object
// we are talking about.  Note the values are sequential -- the item will
// be exactly one of these things
//

#define INST_TYP_FUNCTION     0x01
#define INST_TYP_LABEL        0x02
#define INST_TYP_PARAMETER    0x03
#define INST_TYP_VARIABLE     0x04
#define INST_TYP_CONSTANT     0x05
#define INST_TYP_MACRO        0x06
#define INST_TYP_TYPEDEF      0x07
#define INST_TYP_STRUCNAM     0x08
#define INST_TYP_ENUMNAM      0x09
#define INST_TYP_ENUMMEM      0x0A
#define INST_TYP_UNIONNAM     0x0B
#define INST_TYP_SEGMENT      0x0C
#define INST_TYP_GROUP        0x0D
#define INST_TYP_PROGRAM      0x0E
#define INST_TYP_CLASSNAM     0x0F
#define INST_TYP_MEMFUNC      0x10
#define INST_TYP_MEMVAR       0x11

// this is the attribute part of the field, it describes the storage
// class and/or scope of the instance.  Any combination of the bits
// might be set by some language compiler, but there are some combinations
// that done make sense.

#define INST_ATR_LOCAL       0x001
#define INST_ATR_STATIC      0x002
#define INST_ATR_SHARED      0x004
#define INST_ATR_NEAR        0x008
#define INST_ATR_COMMON      0x010
#define INST_ATR_DECL_ONLY   0x020
#define INST_ATR_PUBLIC      0x040
#define INST_ATR_NAMED       0x080
#define INST_ATR_MODULE      0x100
#define INST_ATR_VIRTUAL     0x200
#define INST_ATR_PRIVATE     0x400
#define INST_ATR_PROTECT     0x800

#define IMODE_VIRTUAL	     0x001
#define IMODE_PRIVATE	     0x002
#define IMODE_PUBLIC	     0x004
#define IMODE_PROTECT	     0x008

// simple minded printf replacements, only %d, %s supported -- SMALL

VOID BSC_API BSCFormat(LPCH lpchOut, LSZ lszFormat, va_list va);
VOID BSC_API BSCSprintf(LPCH lpchOut, LSZ lszFormat, ...);
VOID BSC_API BSCPrintf(LSZ lszFormat, ...);


#ifdef DOSX32
#define BASED_CODE
#else
#define BASED_CODE _based(_segname("_CODE")) 
#endif
